import paramiko
import time

class RemoteTester:

    def __init__(password):
        self.ssh = paramiko.SSHClient()
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.ssh.connect("mp_ubuntu.andrew.cmu.edu", username="mpubuntu", password=password, port="22")
        time.sleep(5)

    def exec_ssh(self, cmd, ret = False):
        stdin, stdout, stderr = self.ssh.exec_command(cmd)
        if not ret:
            [print(i[:-1]) for i in stdout.readlines()]
            [print(i[:-1]) for i in stderr.readlines()]
        return stdout.readlines()

    def exec_visa(self, params, ret = False):
        return exec_ssh(f"python3 /home/mpubuntu/projects/i22j_test/equipment_control.py {params}", ret = ret)



class Config:
    running_pynq = False
    running_cocotb = not running_pynq
    inv_outs = running_pynq
    _ns = 1000
    T_default = int(10 * _ns)

    def aslist(generator):
        "Function decorator to transform a generator into a list"
        def wrapper(*args, **kwargs):
            try:
                next(generator(*args, **kwargs))
            except StopIteration as ex:
                return ex.value
        return wrapper

    if running_cocotb:
        import cocotb
        default_decorator = cocotb.coroutine
    else:
        default_decorator = aslist



from .config import Config
import random, copy, math, time
from .binary import *
from .essentials import *

if Config.running_cocotb:
    import cocotb
    from cocotb.triggers import Timer
    from cocotb.result import TestFailure
    from cocotb.binary import BinaryValue
    from cocotb.clock import Clock
    from cocotb.triggers import FallingEdge, RisingEdge


class Scanner:

    def __init__(self, T = Config.T_default, interface = None, scan_clk = None, scan_in = None, scan_out = None,
        scan_en = None, scan_wen = None, fromMSB = False, verbosity = 0, clk_running = False,
        clk_b = False, length = None, read_func = read_signal):

        if interface is None:
            self._scan_in = scan_in
            self._scan_clk = scan_clk
            self._scan_en = scan_en
            self._scan_out = scan_out
            self._scan_wen = scan_wen
        else:
            self._scan_in = interface.get('scan_in')
            self._scan_clk = interface.get('scan_clk')
            self._scan_en = interface.get('scan_en')
            self._scan_out = interface.get('scan_out')
            self._scan_wen = interface.get('scan_wen')

        self.T = T
        self.fromMSB = fromMSB
        self.verbosity = verbosity
        self.clk_running = clk_running
        self.clk_b = clk_b
        self.length = length
        self.read_func = read_func

        self.halfT = int(T/2)
        self.quarterT = int(T/4)

    def scan_in(self, val, T = None, scan_clk = None, scan_in = None, scan_out = None,
        scan_en = None, scan_wen = None, fromMSB = None, verbosity = None, clk_running = None,
        clk_b = None, read_func = None):

        if T is None:
            T = self.T
            halfT = self.halfT
            quarterT = self.quarterT
        else:
            halfT = int(T/2)
            quarterT = int(T/4)

        if scan_clk is None:
            scan_clk = self._scan_clk
        if scan_in is None:
            scan_in = self._scan_in
        if scan_out is None:
            scan_out = self._scan_out
        if scan_en is None:
            scan_en = self._scan_en
        if scan_wen is None:
            scan_wen = self._scan_wen
        if fromMSB is None:
            fromMSB = self.fromMSB
        if verbosity is None:
            verbosity = self.verbosity
        if clk_running is None:
            clk_running = self.clk_running
        if clk_b is None:
            clk_b = self.clk_b
        if read_func is None:
            read_func = self.read_func

        if self.length is not None and len(val) != self.length:
            print(f'Warning: number of scanned bits ({len(val)}) is not equal to the scan chain length ({self.length})')

        if fromMSB:
            val = val[::-1]

        if Config.running_cocotb:
            if clk_running:
                if clk_b:
                    yield RisingEdge(scan_clk)
                else:
                    yield FallingEdge(scan_clk)
            else:
                yield Timer(halfT)

        if Config.running_cocotb:
            if scan_en is not None:
                if read_func(scan_en) == '1':
                    print('ERROR: scan enable is high, another chain might be using the scan ports')
                    return None
                else:
                    write_signal(scan_en, 1)

        outstr = ''
        for i, v in enumerate(val):
            if scan_out is not None:
                outstr = outstr + read_func(scan_out)

            if (verbosity == 1 and i%100 == 0) or (verbosity == 2 and i%10 == 0) or (verbosity > 2):
                print(f"Scanning bit {i} out of {len(val)-1}...")

            write_signal(scan_in, int(v, 2))

            if Config.running_cocotb:
                yield cycle(scan_clk, T, clk_running)
            else:
                cycle(scan_clk, T, clk_running)

        if fromMSB:
            outstr = outstr[::-1]

        if scan_en is not None:
            write_signal(scan_en, 0)

        if scan_wen is not None:
            write_signal(scan_wen, 1)
            if Config.running_cocotb:
                yield cycle(scan_clk, T, clk_running)
            else:
                cycle(scan_clk, T, clk_running)
            write_signal(scan_wen, 0)

        if Config.inv_outs:
            outstr = inv(outstr)

        return outstr

    def scan_out(self, num = None, T = None, scan_clk = None, scan_in = None, scan_out = None,
        scan_en = None, fromMSB = None, verbosity = None, clk_running = None, clk_b = None):

        if num is None:
            if self.length is None:
                print('ERROR: Please either set the scan chain length or give the number of bits')
            else:
                num = self.length

        if Config.running_cocotb:
            out = yield self.scan_in(''.zfill(num), T = T, scan_clk = scan_clk, scan_in = scan_in, scan_out = scan_out,
                scan_en = scan_en, fromMSB = fromMSB, verbosity = verbosity, clk_running = clk_running, clk_b = clk_b)
        else:
            out = self.scan_in(''.zfill(num), T = T, scan_clk = scan_clk, scan_in = scan_in, scan_out = scan_out,
                scan_en = scan_en, fromMSB = fromMSB, verbosity = verbosity, clk_running = clk_running, clk_b = clk_b)

        return out

    scan_in = Config.default_decorator(scan_in)
    scan_out = Config.default_decorator(scan_out)


class JTAG:

    def __init__(self, chains, confouts, interface = None, T = Config.T_default, jtag_clk = None, jtag_in = None,
                jtag_out = None, jtag_en = None, jtag_load = None, scan_clk = None, scan_in = None, scan_out = None,
                scan_en = None, fromMSB = False, read_func = read_signal):

        if hasattr(chains, '__iter__'):
            self.chains = chains
            self.num_chains = len(chains)
        else:
            self.chains = []
            self.num_chains = chains

        self.confouts = confouts
        self.num_confouts = len(confouts)
        self.read_func = read_func

        if interface is None:
            self._jtag_in = jtag_in
            self._jtag_clk = jtag_clk
            self._jtag_load = jtag_load
            self._jtag_out = jtag_out
            self._jtag_en = jtag_en
            self._scan_in = scan_in
            self._scan_clk = scan_clk
            self._scan_en = scan_en
            self._scan_out = scan_out
        else:
            self._jtag_in = interface['jtag_in']
            self._jtag_clk = interface['jtag_clk']
            self._jtag_load = interface['jtag_load']
            self._jtag_out = interface['jtag_out']
            self._jtag_en = interface['jtag_en']
            self._scan_in = interface['scan_in']
            self._scan_clk = interface['scan_clk']
            self._scan_en = interface['scan_en']
            self._scan_out = interface['scan_out']

        self.T = T
        self.halfT = int(T/2)
        self.fromMSB = fromMSB

        self.num_chain_bits = log2up(self.num_chains)
        self.current_chain_bits = ''.zfill(self.num_chain_bits)
        self.current_confouts = ''.zfill(self.num_confouts)
        self.width = self.num_chain_bits + self.num_confouts

        self.jtag_scan_chain = Scanner(T = T, scan_clk = jtag_clk, scan_in = jtag_in, scan_out = jtag_out,
            scan_en = jtag_en, scan_wen = jtag_load, fromMSB = fromMSB, verbosity = 0, clk_running = False,
            clk_b = False, length = self.width, read_func = self.read_func)

    def set_chain(self, chain):
        if isinstance(chain, Scanner):
            chain_num = self.chains.index(chain)
        else:
            chain_num = chain

        assert chain_num < self.num_chains, 'Chain number greater than the number of chains in JTAG!'

        selected_chain_bits = bin(chain_num)[2:].zfill(self.num_chain_bits)
        val = self.current_confouts + selected_chain_bits

        assert(len(val) == self.width)

        if Config.running_cocotb:
            outstr = yield self.jtag_scan_chain.scan_in(val)
        else:
            outstr = self.jtag_scan_chain.scan_in(val)
        self.current_chain_bits = selected_chain_bits

        return outstr

    def set_confouts(self, confouts):
        confs = list(self.current_confouts)[::-1]
        for key, value in confouts.items():
            assert key in self.confouts, f'The config signal {key} not defined in the JTAG initialization.'
            confs[self.confouts.index(key)] = value

        val = ''.join(confs[::-1]) + self.current_chain_bits
        assert(len(val) == self.width)

        if Config.running_cocotb:
            outstr = yield self.jtag_scan_chain.scan_in(val)
        else:
            outstr = self.jtag_scan_chain.scan_in(val)
        self.current_confouts = ''.join(confs[::-1])

        return outstr

    def scan_in(self, val):
        chain = int(self.current_chain_bits, 2)
        clk = None
        if self.chains[chain]._scan_clk is None:
            clk = self._scan_clk

        if Config.running_cocotb:
            out = yield self.chains[chain].scan_in(val, T = self.T, scan_in = self._scan_in, scan_en = self._scan_en, scan_out = self._scan_out, scan_clk = clk, read_func = self.read_func)
        else:
            out = self.chains[chain].scan_in(val, T = self.T, scan_in = self._scan_in, scan_en = self._scan_en, scan_out = self._scan_out, scan_clk = clk, read_func = self.read_func)

        return out

    def scan_out(self, num = None):
        chain = int(self.current_chain_bits, 2)
        clk = None
        if self.chains[chain]._scan_clk is None:
            clk = self._scan_clk

        if Config.running_cocotb:
            out = yield self.chains[chain].scan_out(num, T = self.T, scan_in = self._scan_in, scan_en = self._scan_en, scan_out = self._scan_out, scan_clk = clk)
        else:
            out = self.chains[chain].scan_out(num, T = self.T, scan_in = self._scan_in, scan_en = self._scan_en, scan_out = self._scan_out, scan_clk = clk)

        return out

    def select_and_scan_in(self, chain, val):
        if Config.running_cocotb:
            yield self.set_chain(chain)
            out = yield self.scan_in(val)
        else:
            self.set_chain(chain)
            out = self.scan_in(val)
        return out

    def select_and_scan_out(self, chain, num = None):
        if Config.running_cocotb:
            yield self.set_chain(chain)
            out = yield self.scan_out(num)
        else:
            self.set_chain(chain)
            out = self.scan_out(num)
        return out

    def self_test(self):
        in_chain = rand_binary_str(self.num_chain_bits)
        while int(in_chain, 2) >= self.num_chains:
            in_chain = rand_binary_str(self.num_chain_bits)
        in_confouts = rand_binary_str(self.num_confouts)
        ins = in_confouts + in_chain

        scan_wen = self.jtag_scan_chain._scan_wen
        self.jtag_scan_chain._scan_wen = None
        if Config.running_cocotb:
            prev = yield self.jtag_scan_chain.scan_in(ins, scan_wen = None)
            out = yield self.jtag_scan_chain.scan_in(ins, scan_wen = None)
        else:
            prev = self.jtag_scan_chain.scan_in(ins, scan_wen = None)
            out = self.jtag_scan_chain.scan_in(ins, scan_wen = None)

        self.jtag_scan_chain._scan_wen = scan_wen

        if ins != out:
            print(f'ERROR: JTAG chain failed self test! in: {ins}, out: {out}')
        else:
            print('JTAG chain: pass')

        for i, chain in enumerate(self.chains):
            if chain.length is None:
                print(f'Warning: The length of chain {i} is unknown!')
            else:
                ins = rand_binary_str(chain.length)
                if Config.running_cocotb:
                    prev = yield self.select_and_scan_in(chain, ins)
                    out = yield self.select_and_scan_out(chain)
                else:
                    prev = self.select_and_scan_in(chain, ins)
                    out = self.select_and_scan_out(chain)

                if ins != out:
                    print(f'ERROR: Chain {i} failed JTAG self test!')
                    print(f'Expected: {ins}')
                    print(f'Obtained: {out}')
                else:
                    print(f'Scan chain {i}: pass')

    set_confouts = Config.default_decorator(set_confouts)
    set_chain = Config.default_decorator(set_chain)
    scan_in = Config.default_decorator(scan_in)
    scan_out = Config.default_decorator(scan_out)
    select_and_scan_in = Config.default_decorator(select_and_scan_in)
    select_and_scan_out = Config.default_decorator(select_and_scan_out)
    self_test = Config.default_decorator(self_test)


if Config.running_pynq:

    class MMIOScanner:
        def __init__(self, T = Config.T_default, interface = None, fromMSB = False):
            self.T = T
            self.interface = interface
            self.fromMSB = fromMSB
            self.num_bits = None
            self.num_rows = None

        def reset(self):
            self.interface['reset'].on()
            self.interface['valid'].off()
            self.interface['num_bits'].write(0, int('001111111111111111', 2))
            time.sleep(0.01)
            self.interface['reset'].off()

        def set_data(self, data):
            self.num_bits = len(data)
            self.num_rows = math.ceil(num_bits/32)
            slack = num_rows*32 - self.num_bits
            assert(self.num_rows <= 2048)

            if self.fromMSB:
                data = data[::-1]

            for i in range(num_rows-1):
                self.interface['mmio_scan_in'].write(offset=i*4, data = int(data[32*i:32*(i+1)][::-1], 2))

            last_line = int(data[32*(self.num_rows-1):][::-1].zfill(32), 2)
            self.interface['mmio_scan_in'].write(offset=(num_rows-1)*4, data=last_line)
            self.interface['num_bits'].write(self.num_bits, int('001111111111111111', 2))

        def scan_in(self, data = None):
            if data is not None:
                self.set_data(data)
            self.interface['valid'].on()
            time.sleep(0.001)
            self.interface['valid'].off()
            rising_edge(self.interface['ready'])

        def scan_out(self):
            out_data = ''
            array = list(self.interface['mmio_scan_out'].array)
            for i in range(num_rows-1):
                out_data = out_data + bin(array[(i+1)%2048])[2:].zfill(32)[::-1]

            last_line = bin(array[self.num_rows%2048])[2:].zfill(32)[::-1][slack:]
            out_data = out_data + last_line

            if self.fromMSB:
                out_data = out_data[::-1]

            return out_data



import random, copy, math, collections

def rm_header(b):
    if b.startswith("0b"):
        return b[2:]
    else:
        return b

def onehot(val, num_bits):
    if is_bin(val):
        val = int(val, 2)

    assert val <= num_bits, 'Val must be less than or equal to num_bits'

    out = ['0'] * num_bits
    out[val] = '1'
    return ''.join(out[::-1])

def bitblast(bits):
    return list(bits)[::-1]

def bitmerge(arr):
    return ''.join(arr[::-1])

def hex2bin(hex_val):
    bin_val = ''

    for char in hex_val:
        if char in ['x', 'X']:
            bin_val = bin_val + 'x'
        elif char in ['z', 'Z']:
            bin_val = bin_val + 'z'
        else:
            bin_val = bin_val + bin(int(char, 16))[2:].zfill(4)

    return bin_val.upper()

def bin2hex(bin_val):
    hex_val = ''
    num_hex_digits = math.ceil(len(bin_val)/4)
    bin_val = bin_val.zfill(num_hex_digits*4)

    for i in range(0, len(bin_val), 4):
        bin_quad = bin_val[i:i+4]

        if 'x' in bin_quad or 'X' in bin_quad:
            hex_val = hex_val + 'x'
        elif 'z' in bin_quad or 'Z' in bin_quad:
            hex_val = hex_val + 'z'
        else:
            hex_val = hex_val + hex(int(bin_val[i:i+4], 2))[2:]

    return hex_val.upper()

def as_bin(n, width = None):
    out = rm_header(bin(int(n)))
    if width:
        out = out.zfill(width)
    return out

def is_bin(n):
    return isinstance(n, str) and all([bit in ['0', '1'] for bit in n])

def as_int(b, invalid = -9999):
    if b.startswith("b'"):
        b = b[2:]

    if all([bit in ['0', '1'] for bit in b]):
        return int(b, 2)
    else:
        return invalid

def as_sint(b, invalid = -9999):
    b = rm_header(b)

    if all([bit in ['0', '1'] for bit in b]):
        if b[0] == '1':
            return (int(inv(b), 2) + 1) * -1
        else:
            return int(b, 2)
    else:
        return invalid

def inv(n):
    n = rm_header(n)
    out = ''
    for bit in n:
        if bit == '1':
            out = out + '0'
        elif bit == '0':
            out = out + '1'
        else:
            print('inv: Not a binary string!')
    return out

def onehot(n, width):
    out = ['0'] * width
    out[n] = '1'
    return ''.join(out[::-1])

def randbits(length):
    out = ''
    for i in range(length):
        out = out + str(random.randint(0, 1))

    return out

def log2up(n):
    if n == 0:
        return 0
    elif n == 1:
        return 1
    else:
        return math.ceil(math.log2(n))

def diff(a, b):
    if(len(a) != len(b)):
        print('lengths are different')
        l = min(len(a), len(b))
    else:
        l = len(a)

    for i in range(l):
        if(a[i] != b[i]):
            print(i, a[i], b[i])

def rotating_equals(a, b):
    length = len(a)
    b = collections.deque(b)
    a = collections.deque(a)
    for i in range(1, length):
        if list(a) == list(b):
            return True
        b.rotate(1)

    return False



from .config import Config
import random, copy, math, time
from .binary import *

if Config.running_cocotb:
    import cocotb
    from cocotb.triggers import Timer
    from cocotb.result import TestFailure
    from cocotb.binary import BinaryValue
    from cocotb.clock import Clock
    from cocotb.triggers import FallingEdge, RisingEdge

def read_signal(obj):
    if Config.running_cocotb:
        return str(obj)
    elif Config.running_pynq:
        return str(obj.read())

def write_signal(obj, val):
    if Config.running_cocotb:
        obj <= val
    elif Config.running_pynq:
        obj.write(val)

def cycle(clock, T = Config.T_default, clk_running = False):
    if Config.running_cocotb:
        if clk_running:
            yield RisingEdge(clock)
            yield FallingEdge(clock)
        else:
            halfT = int(T/2)
            yield Timer(halfT)
            clock <= int(not (int(clock)))
            yield Timer(halfT)
            clock <= int(not (int(clock)))
    elif Config.running_pynq:
        clock.on()
        clock.off()

def cycles(clock, n, T = Config.T_default, clk_running = False):
    if Config.running_cocotb:
        for i in range(n):
            yield cycle(clock, T, clk_running)
    elif Config.running_pynq:
        for i in range(n):
            cycle(clock, T, clk_running)

def wait_for_val(signal, val, clock, clk_running = True, T = Config.T_default, timeout = 5000, read_func = read_signal):
    for i in range(timeout):
        if(read_func(signal) == val):
            return
        else:
            if Config.running_cocotb:
                yield cycle(clock, T)
            elif Config.running_pynq:
                cycle(clock, T)

    assert False, 'Wait timed out!'

# These two functions are only for pynq
if Config.running_pynq:
    def rising_edge(signal):
        while(signal.read() == 0):
            time.sleep(0.001)

    def falling_edge(signal):
        while(signal.read() == 1):
            time.sleep(0.001)

if Config.running_cocotb:
    cycle = Config.default_decorator(cycle)
    cycles = Config.default_decorator(cycles)
    wait_for_val = Config.default_decorator(wait_for_val)



from telegram.ext import Updater, MessageHandler, Filters
import sys

class TelegramNotifier():
    def __init__(self, token, chat_id):
        self.token = str(token)
        self.chat_id = str(chat_id)
        self.updater = Updater(token = self.token, use_context = True)
        self.bot = self.updater.dispatcher.bot

    def notify(self, msg:str)->None:
        self.bot.send_message(chat_id = self.chat_id, text = msg)

def telegram_notify(token, chat_id, msg = "Notification"):
    tn = TelegramNotifier(token, chat_id)
    tn.notify(msg)



import random, copy, math
from .binary import *
from .scan_driver import Scanner
from .config import Config
from .essentials import *

if Config.running_cocotb:
    import cocotb

efpga_reset
efpga_jtag_clk
efpga_jtag_load
efpga_jtag_in
efpga_jtag_out

efpga_scan_clk
efpga_scan_en
efpga_scan_in
efpga_scan_out

conf_sel[1:0]

conf_out[3:0]

gnd:
conf_sel[3:2]


class Clkgen:

    def __init__(self, chain):
        self.interface = interface
        self.chain = chain

    def set_fast_clk(division = 7, oscillator = 1, block = None):
        if block is None:
            blk = '111111'
        else:
            blk = bin(2 ** block)[2:].zfill(6)
        div = bin(2 ** division)[2:].zfill(14)
        osc = bin(2 ** oscillator)[2:].zfill(12)
        bits = blk + div + osc

        write_signal(interface['sel_clk_extern'], 1)
        write_signal(interface['global_en'], 0)
        write_signal(interface['en_common'], 0)

        if Config.running_cocotb:
            yield self.chain.scan_in(bits)
            out = yield self.chain.scan_out(bits)
        elif Config.running_pynq:
            self.chain.scan_in(bits)
            out = self.chain.scan_out(bits)
        assert out == bits, out

        write_signal(interface['global_en'], 1)
        write_signal(interface['en_common'], 1)
        write_signal(interface['sel_clk_extern'], 0)


    def set_slow_clk():
        write_signal(interface['sel_clk_extern'], 1)
        write_signal(interface['global_en'], 0)
        write_signal(interface['en_common'], 0)

        bits = ''.zfill(6+14+12)

        if Config.running_cocotb:
            yield self.chain.scan_in(bits)
            out = yield self.chain.scan_out(bits)
        elif Config.running_pynq:
            self.chain.scan_in(bits)
            out = self.chain.scan_out(bits)
        assert out == bits, out

    set_fast_clk = Config.default_decorator(set_fast_clk)
    set_slow_clk = Config.default_decorator(set_slow_clk)



from time import sleep
import os, time
import pyvisa
import sys

rm=pyvisa.ResourceManager()
supply_dummy_obfus = rm.open_resource('GPIB0::4::INSTR')
supply_fpga_sensor = rm.open_resource('GPIB0::7::INSTR')
supply_io_top = rm.open_resource('GPIB0::5::INSTR')
func_gen = rm.open_resource('GPIB0::10::INSTR')
osc = rm.open_resource('USB0::62700::60984::SDSMMEBC4R1071::0::INSTR')


def autoset():
    osc.write('ASET')

def osc_measure_freq(channel = 1):
    channel = int(channel)
    print(f"FREQ: {float(osc.query(f'C{channel}:PAVA? FREQ')[13:-3])}")
    print(f"PER: {float(osc.query(f'C{channel}:PAVA? PER')[12:-2])}")

    return float(osc.query(f'C{channel}:PAVA? FREQ')[13:-3])


def set_voltage(supply, channel, voltage):
    if isinstance(supply, str):
        supply = globals()[supply]
    channel = int(channel)
    voltage = float(voltage)
    assert(voltage <= 1)

    supply.write('INST:SEL OUT' + str(channel))
    supply.write('VOLT ' + str(voltage))
    print("VOLTAGE: " + str(float(supply.query("VOLT?"))))
    print("CURRENT: " + str(float(supply.query("CURR?"))))

def turn_on(supply):
    if isinstance(supply, str):
        supply = globals()[supply]

    if supply == func_gen:
        func_gen.write('OUTP1:POS ON')
    else:
        supply.write('OUTP ON')
        time.sleep(0.2)
        print('Channel 1 I, V: ' + supply.query('MEAS:CURR?;VOLT?'))

def query_voltage(supply, chan=1):
    if isinstance(supply, str):
        supply = globals()[supply]
    chan = int(chan)

    supply.write('INST:SEL OUT' + str(chan))
    print(float(supply_fpga_sensor.query("VOLT?")))


def turn_off(supply):
    if isinstance(supply, str):
        supply = globals()[supply]

    if supply == func_gen:
        func_gen.write('OUTP1:POS OFF')
    else:
        supply.write('OUTP OFF')
        time.sleep(0.2)
        print(supply.query('MEAS:CURR?;VOLT?'))

def set_freq(freq):
    freq = float(freq)
    func_gen.write('FREQ ' + str(freq) + ' MHz')
    time.sleep(1)
    func_gen.query('FREQ?')

def init_test(vcc=0.85, vdd_core=0.7):
    vcc = float(vcc)
    vdd_core = float(vdd_core)

    # Init power supplies
    supply_fpga_sensor.write('OUTP OFF')
    supply_dummy_obfus.write('OUTP OFF')
    supply_io_top.write('OUTP OFF')

    supply_io_top.write('INST:SEL OUT1')
    supply_io_top.write('CURRENT 0.5')
    supply_io_top.write('VOLT 1.8')
    supply_io_top.write('INST:SEL OUT2')
    supply_io_top.write('CURRENT 0.5')
    supply_io_top.write('VOLT ' + str(vcc))

    #vdd_test = 0.7
    #vdd_core = 0.7
    supply_dummy_obfus.write('INST:SEL OUT1')
    supply_dummy_obfus.write('CURRENT 0.1')
    supply_dummy_obfus.write('VOLT 0.0')
    supply_dummy_obfus.write('INST:SEL OUT2')
    supply_dummy_obfus.write('CURRENT 0.1')
    supply_dummy_obfus.write('VOLT 0.0')
    supply_fpga_sensor.write('INST:SEL OUT1')
    supply_fpga_sensor.write('CURRENT 0.25')
    supply_fpga_sensor.write('VOLT 0.0')
    supply_fpga_sensor.write('INST:SEL OUT2')
    supply_fpga_sensor.write('CURRENT 0.1')
    supply_fpga_sensor.write('VOLT 0.0')

    # Func gen
    #func_gen.write('OUTP1 OFF')
    #func_gen.write('OUTP2 OFF')
    #func_gen.write('FUNC:MOD1 SQUAre')
    #func_gen.write('OUTP1:DIV 1')
    #func_gen.write('PM1 OFF')
    #func_gen.write('DEL1 0ns')
    #func_gen.write('VOLT1:LOW 0mV')
    #func_gen.write('VOLT1:HIGH 1.6V')
    #func_gen.write('VOLT1:TERM 0V')
    #func_gen.write('FREQ 20 MHz')
    #time.sleep(1)
    #func_gen.query('FREQ?')
    #func_gen.write('OUTP1:POS ON')

def turn_off_all():
    supply_fpga_sensor.write('OUTP OFF')
    print("CORE VDD OFF")
    print(supply_fpga_sensor.query('MEAS:CURR?;VOLT?'))
    supply_dummy_obfus.write('OUTP OFF')
    print("TEST VDD OFF")
    print(supply_dummy_obfus.query('MEAS:CURR?;VOLT?'))
    supply_io_top.write('OUTP OFF')
    print("IO VDDs OFF")
    print(supply_io_top.query('MEAS:CURR?;VOLT?'))

def turn_on_top():
    supply_io_top.write('OUTP ON')
    print("IO VDDs")
    print(supply_io_top.query('MEAS:CURR?;VOLT?'))

def turn_on_core(core, vcc_core = 0.7):
    vcc_core = float(vcc_core)
    if core == 'efpga':
        supply_fpga_sensor.write('INST:SEL OUT1')
        supply_fpga_sensor.write('CURRENT 0.1')
        supply_fpga_sensor.write('VOLT ' + str(vcc_core))
        supply_fpga_sensor.write('OUTP ON')
        print("CORE VDD ON")
        print(supply_fpga_sensor.query('MEAS:CURR?;VOLT?'))

    elif core == 'sensor':
        supply_fpga_sensor.write('INST:SEL OUT2')
        supply_fpga_sensor.write('CURRENT 0.1')
        supply_fpga_sensor.write('VOLT ' + str(vcc_core))
        supply_fpga_sensor.write('OUTP ON')
        print("CORE VDD ON")
        print(supply_fpga_sensor.query('MEAS:CURR?;VOLT?'))

    elif core == 'dummy':
        supply_dummy_obfus.write('INST:SEL OUT1')
        supply_dummy_obfus.write('CURRENT 0.1')
        supply_dummy_obfus.write('VOLT ' + str(vcc_core))
        supply_dummy_obfus.write('OUTP ON')
        print("CORE VDD ON")
        print(supply_fpga_sensor.query('MEAS:CURR?;VOLT?'))

    elif core == 'obfus':
        supply_dummy_obfus.write('INST:SEL OUT2')
        supply_dummy_obfus.write('CURRENT 0.1')
        supply_dummy_obfus.write('VOLT ' + str(vcc_core))
        supply_dummy_obfus.write('OUTP ON')
        print("CORE VDD ON")
        print(supply_fpga_sensor.query('MEAS:CURR?;VOLT?'))


def measure_all():
    print('FPGA')
    supply_io_top.write('INST:SEL OUT1')
    val = supply_fpga_sensor.query('MEAS:CURR?;VOLT?')
    print( "CUR : " + str(round(float(val.split(";")[0]),4)) )
    print( "VOLT: " + str(round(float(val.split(";")[1]),4)) )

    print('\nDUMMY')
    supply_dummy_obfus.write('INST:SEL OUT1')
    val = supply_dummy_obfus.query('MEAS:CURR?;VOLT?')
    print( "CUR : " + str(round(float(val.split(";")[0]),4)) )
    print( "VOLT: " + str(round(float(val.split(";")[1]),4)) )

    print('\nIO')
    supply_io_top.write('INST:SEL OUT1')
    val = supply_io_top.query('MEAS:CURR?;VOLT?')
    print( "CUR : " + str(round(float(val.split(";")[0]),4)) )
    print( "VOLT: " + str(round(float(val.split(";")[1]),4)) )

    print('\nCLKGEN')
    supply_io_top.write('INST:SEL OUT2')
    val = supply_io_top.query('MEAS:CURR?;VOLT?')
    print( "CUR : " + str(round(float(val.split(";")[0]),4)) )
    print( "VOLT: " + str(round(float(val.split(";")[1]),4)) )


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Please call this script with the name of the function and the parameters you want to pass")
    else:
        func = sys.argv[1]
        args = sys.argv[2:]
        locals()[func](*args)
